#!/bin/python
import roomai

class DouDiZhuPokerMaxPlayer(roomai.utils.AbstractPlayer):
    
    def receiveInformation(self,info):
        return 0;

    def receiveActions(self, actions):
        return 0;

    def takeAction(self, history):
        return 0;
